@@ -15,8 +15,6 @@ import numpy
 #numpy.seterr(all='ignore')
 
 import warnings
-import logging
-LOG = logging.getLogger('cogent.align')
 
 from cogent.align.traceback import alignment_traceback
 from cogent.evolve.likelihood_tree import LikelihoodTreeEdge
@@ -29,12 +27,12 @@ from cogent.util import parallel
 from cogent.util.modules import importVersionedModule, ExpectedImportError
 try:
     pyrex_align_module = importVersionedModule('_pairwise_pogs', globals(),
-            (3, 0), LOG, "slow Python alignment implementation")
+            (3, 0), "slow Python alignment implementation")
 except ExpectedImportError:
     pyrex_align_module = None
 try:
     pyrex_seq_align_module = importVersionedModule('_pairwise_seqs', globals(),
-            (3, 0), LOG, "slow Python alignment implementation")
+            (3, 0), "slow Python alignment implementation")
 except ExpectedImportError:
     pyrex_seq_align_module = None
 
@@ -42,7 +40,7 @@ __author__ = "Peter Maxwell"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Peter Maxwell"
 __email__ = "pm67nz@gmail.com"
 __status__ = "Production"
@@ -623,7 +621,7 @@ class PairEmissionProbs(object):
     def __getitem__(self, index):
         assert len(index) == 2, index
         return PairEmissionProbs(self.pair[index], self.bins)
-                
+    
     def hirschberg(self, TM, dp_options):
         """linear-space alignment algorithm
         A linear space algorithm for computing maximal common subsequences.
@@ -723,7 +721,7 @@ class PairEmissionProbs(object):
             else:
                 msg = 'dp'
             if memory > 500:
-                LOG.warn('%s will use > %sMb.' % (msg, memory))
+                warnings.warn('%s will use > %sMb.' % (msg, memory))
             track = encoder.getEmptyArray(problem_dimensions)
         else:
             track = encoder = None
