@@ -16,7 +16,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Micah Hamady", "Zongzhi Liu", "Mike Robeson",
        "Catherine Lozupone", "Rob Knight", "Daniel McDonald", "Jeremy Widmann"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Micah Hamady"
 __email__ = "hamady@colorado.edu"
 __status__ = "Prototype"
@@ -608,13 +608,22 @@ def build_tree_from_alignment(aln, moltype, best_tree=False, params=None):
     app.Parameters['-tree1'].on(get_tmp_filename(app.WorkingDir))
     app.Parameters['-seqtype'].on(moltype.label)
 
-    seq_collection = SequenceCollection(aln)
+    seq_collection = SequenceCollection(aln, MolType=moltype)
+
+    #Create mapping between abbreviated IDs and full IDs
+    int_map, int_keys = seq_collection.getIntMap()
+    #Create SequenceCollection from int_map.
+    int_map = SequenceCollection(int_map,MolType=moltype)
+
 
     # Collect result
-    result = app(seq_collection.toFasta())
+    result = app(int_map.toFasta())
 
     # Build tree
     tree = DndParser(result['Tree1Out'].read(), constructor=PhyloNode)
+    
+    for tip in tree.tips():
+        tip.Name = int_keys[tip.Name]
 
     # Clean up
     result.cleanUp()
