@@ -21,8 +21,6 @@ from cogent.util.array import cartesian_product
 
 import re
 import string
-import logging
-LOG = logging.getLogger('cogent.data')
 from numpy import array, sum, transpose, remainder, zeros, arange, newaxis, \
     ravel, asarray, fromstring, take, uint8, uint16, uint32, take
 from string import maketrans, translate
@@ -35,7 +33,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight",
                     "Andrew Butterfield"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -315,7 +313,7 @@ class Enumeration(tuple):
                 data = ravel(array(a))
             except ValueError: #try mapping to string
                 data = ravel(array(map(str, a)))
-        return sum(asarray(self._allowed_range == data, Int), 1)
+        return sum(asarray(self._allowed_range == data, Int), axis=-1)
     
     def _get_pairs(self):
         """Accessor for pairs, lazy evaluation."""
