@@ -17,7 +17,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight", \
         "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -41,8 +41,6 @@ from random import choice
 
 import re
 import string
-import logging
-LOG = logging.getLogger('cogent.data')
 
 import numpy
 from numpy import array, sum, transpose, remainder, zeros, arange, newaxis, \
