@@ -27,15 +27,14 @@ from numpy.random import permutation
 from operator import eq, ne
 from random import shuffle
 import re
-import logging
-LOG = logging.getLogger('cogent.data')
+import warnings
 
 __author__ = "Rob Knight, Gavin Huttley, and Peter Maxwell"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Gavin Huttley",
                     "Matthew Wakefield", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -685,22 +684,29 @@ class Sequence(_Annotatable, SequenceI):
     def gettype(self):
         """Return the sequence type."""
         
-        return self.MolType._type
+        return self.MolType.label
     
     def resolveambiguities(self):
         """Returns a list of tuples of strings."""
         ambigs = self.MolType.resolveAmbiguity
         return [ambigs(motif) for motif in self._seq]
     
-    def slidingWindows(self, window, step):
+    def slidingWindows(self, window, step, start=None, end=None):
         """Generator function that yield new sequence objects
         of a given length at a given interval.
         Arguments:
             - window: The length of the returned sequence
             - step: The interval between the start of the returned
-              sequence objects"""
-        for pos in range(0, len(self)-window+1,step):
-            yield self[pos:pos+window]
+              sequence objects
+            - start: first window start position
+            - end: last window start position
+        """
+        start = [start, 0][start is None]
+        end = [end, len(self)-window+1][end is None]
+        end = min(len(self)-window+1, end)
+        if start < end and len(self)-end >= window-1:
+            for pos in xrange(start, end, step):
+                yield self[pos:pos+window]
     
     def getInMotifSize(self, motif_length=1, log_warnings=True):
         """returns sequence as list of non-overlapping motifs
@@ -715,7 +721,7 @@ class Sequence(_Annotatable, SequenceI):
             length = len(seq)
             remainder = length % motif_length
             if remainder and log_warnings:
-                LOG.warning('Dropped remainder "%s" from end of sequence' %
+                warnings.warn('Dropped remainder "%s" from end of sequence' %
                         seq[-remainder:])
             return [seq[i:i+motif_length]
                     for i in range(0, length-remainder, motif_length)]
