@@ -28,22 +28,19 @@ Definition of relevant terms or abbreviations:
 from numpy import zeros
 from copy import deepcopy
 import re
-import logging
 from cogent.util.transform import comb
 from cogent.maths.stats.test import correlation
 from operator import or_
 from cogent.util.misc import InverseDict
 from random import shuffle
 
-LOG = logging.getLogger('cogent.tree')
-
 __author__ = "Gavin Huttley, Peter Maxwell and Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Peter Maxwell", "Rob Knight",
                     "Andrew Butterfield", "Catherine Lozupone", "Micah Hamady",
                     "Jeremy Widmann", "Zongzhi Liu", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -1048,10 +1045,12 @@ class TreeNode(object):
         """A new instance of a sub tree that contains all the otus that are
         listed in name_list.
         """
-        edge_names = self.getNodeNames(includeself=1, tipsonly=False)
-        for name in name_list:
-            if name not in edge_names and not ignore_missing:
-                raise ValueError("edge %s not found in tree" % name)
+        edge_names = set(self.getNodeNames(includeself=1, tipsonly=False))
+        if not ignore_missing:
+            # this may take a long time
+            for name in name_list:
+                if name not in edge_names:
+                    raise ValueError("edge %s not found in tree" % name)
         
         new_tree = self._getSubTree(name_list)
         if new_tree is None:
@@ -1691,9 +1690,12 @@ class PhyloNode(TreeNode):
         """
         # max_dist, tip_names = tree.maxTipTipDistance()
         # this is slow
-    
+
+
         max_dist, tip_names = self.maxTipTipDistance()
         half_max_dist = max_dist/2.0
+        if max_dist == 0.0: # only pathological cases with no lengths
+            return self.unrootedDeepcopy()
         # print tip_names
         tip1 = self.getNodeMatchingName(tip_names[0])
         tip2 = self.getNodeMatchingName(tip_names[1])
@@ -1703,16 +1705,17 @@ class PhyloNode(TreeNode):
         else:
             climb_node = tip2
         
-        dist_climbed = climb_node.Length
-        while dist_climbed <= half_max_dist:
-            climb_node = climb_node.Parent
+        dist_climbed = 0.0
+        while dist_climbed + climb_node.Length < half_max_dist:
             dist_climbed += climb_node.Length
-    
-        dist_climbed -= climb_node.Length # dist is now dist from tip to climb_node
-        
-        # now midpt is either at climb_node or on the branch to its parent
+            climb_node = climb_node.Parent
+            
+        # now midpt is either at on the branch to climb_node's  parent
+        # or midpt is at climb_node's parent
         # print dist_climbed, half_max_dist, 'dists cl hamax'
-        if dist_climbed == half_max_dist:
+        if dist_climbed + climb_node.Length == half_max_dist:
+            # climb to midpoint spot
+            climb_node = climb_node.Parent
             if climb_node.isTip():
                 raise RuntimeError('error trying to root tree at tip')
             else:
@@ -1720,6 +1723,7 @@ class PhyloNode(TreeNode):
                 return climb_node.unrootedDeepcopy()
         
         else:
+            # make a new node on climb_node's branch to its parent
             old_br_len = climb_node.Length
             new_root = type(self)()
             new_root.Parent = climb_node.Parent
