@@ -9,7 +9,7 @@ __author__ = "Hua Ying"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Hua Ying"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Hua Ying"
 __email__ = "Hua.Ying@anu.edu.au"
 __status__ = "alpha"
@@ -22,7 +22,7 @@ def location_query(table, query_start, query_end,
     # in which it's being employed
     # should we be setting default values here regarding the columns that start/end
     # are pulled from, or explicitly state which columns
-    if not query:
+    if query is None:
         query = sql.select([table])
     
     if where == 'within':
