@@ -15,7 +15,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -142,11 +142,11 @@ class Genome(object):
             else:
                 descr = gene_table.c.description==Description
         
-        if btype and descr:
+        if btype is not None and descr is not None:
             condition = sql.and_(btype, descr)
-        elif btype:
+        elif btype is not None:
             condition = btype
-        elif descr:
+        elif descr is not None:
             condition = descr
         
         return condition
@@ -184,6 +184,16 @@ class Genome(object):
         return Coordinate(self, CoordName=CoordName, Start=Start, End=End,
                             Strand=Strand, ensembl_coord=ensembl_coord)
     
+    def getGeneByStableId(self, StableId):
+        """returns the gene matching StableId, or None if no record found"""
+        query = self._get_gene_query(self.CoreDb, StableId=StableId)
+        try:
+            record = list(query.execute())[0]
+            gene = Gene(self, self.CoreDb, data=record)
+        except IndexError:
+            gene = None
+        return gene
+    
     def getGenesMatching(self, Symbol=None, Description=None, StableId=None,
                          BioType=None, like=True):
         """Symbol: HGC gene symbol, case doesn't matter
@@ -382,7 +392,10 @@ class Genome(object):
         
         for feature_type in feature_types:
             target_func, target_class = target_coords_funcs[feature_type]
-            db = [self.CoreDb, self.OtherFeaturesDb][feature_type == 'est']
+            db = self.CoreDb
+            if feature_type == 'est':
+                db = self.OtherFeaturesDb
+            
             feature_coords = feature_coord_levels[feature_type].levels
             for feature_coord in feature_coords:
                 chrom_other_coords = get_coord_conversion(coord, feature_coord,
