@@ -4,6 +4,7 @@ import sqlalchemy as sql
 from cogent import DNA
 from cogent.core.annotation import Feature
 from cogent.core.location import Map
+from cogent.util.table import Table
 from cogent.db.ensembl.util import LazyRecord, asserted_one, DisplayString, \
                                     NoItemError
 from cogent.db.ensembl.assembly import Coordinate, CoordSystem, \
@@ -14,7 +15,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -216,7 +217,8 @@ class _StableRegion(GenericRegion):
         super(_StableRegion, self).__init__(genome, db, **kwargs)
     
     def __repr__(self):
-        return '%s(%s; %s)' % (self.Type, self.genome.Species, self.StableId)
+        my_type = self.__class__.__name__
+        return '%s(%s; %s)' % (my_type, self.genome.Species, self.StableId)
     
     def _get_record_for_stable_id(self):
         # subclasses need to provide a function for loading the correct
@@ -294,11 +296,12 @@ class Gene(_StableRegion):
         return '%s(%s)' % (my_type, '; '.join(vals))
     
     def __repr__(self):
+        my_type = self.__class__.__name__
         vals = ['%s=%r' % (key, val) for key, val in self._cached.items() \
                                 if val is not None]
         vals.sort()
         vals.insert(0, 'Species=%r' % self.genome.Species)
-        return '%s(%s)' % (self.Type, '; '.join(vals))
+        return '%s(%s)' % (my_type, '; '.join(vals))
     
     def _get_gene_record(self):
         """adds the gene data to self._table_rows"""
@@ -475,6 +478,42 @@ class Transcript(_StableRegion):
     
     Exons = property(_get_exons)
     
+    def _get_intron_transcript_records(self):
+        if len(self.Exons) < 2:
+            self._set_null_values(["Introns"])
+            return
+        
+        exon_positions = [(exon.Location.Start, exon.Location.End) 
+                            for exon in self.Exons]
+        exon_positions.sort()
+        end = exon_positions[-1][-1]
+        exon_map = Map(locations=exon_positions, parent_length=end)
+        intron_map = exon_map.shadow()
+        
+        intron_positions = [(span.Start, span.End) 
+                            for span in intron_map.spans if span.Start != 0]
+        
+        chrom = self.Location.CoordName
+        strand = self.Location.Strand
+        introns = []
+        rank = 1
+        if strand == -1:
+            intron_positions.reverse()
+        for s, e in intron_positions:
+            coord = self.genome.makeLocation(CoordName=chrom, Start=s, End=e, 
+                                            Strand=strand, ensembl_coord=False)
+            introns.append(Intron(self.genome, self.db, rank, self.StableId,
+                                    coord))
+            rank += 1
+        
+        self._cached['Introns'] = tuple(introns)
+    
+    def _get_introns(self):
+        return self._get_cached_value('Introns', 
+                        self._get_intron_transcript_records)
+    
+    Introns = property(_get_introns)
+    
     def _get_translation_record(self):
         transcript_id = self.transcript_id
         translation_table = self.db.getTable('translation')
@@ -681,6 +720,18 @@ class Transcript(_StableRegion):
             features.append(feature_data)
         return features
     
+    def _get_intron_feature_data(self, parent_map):
+        """return the intron feature data"""
+        features = []
+        if self.Introns is self.NULL_VALUE:
+            return features
+        for intron in self.Introns:
+            feature_data = intron.featureData(parent_map)
+            if feature_data is None:
+                continue
+            features.append(feature_data)
+        return features
+    
     def _get_translated_exon_feature_data(self, parent_map):
         """returns featureD data for translated exons"""
         features = []
@@ -728,6 +779,7 @@ class Transcript(_StableRegion):
         self lies outside parent's span.
         """
         features = self._get_exon_feature_data(parent_map)
+        features += self._get_intron_feature_data(parent_map)
         features += self._get_translated_exon_feature_data(parent_map)
         if self.TranslatedExons:
             features += self._get_Utr_feature_data(parent_map)
@@ -749,7 +801,7 @@ class Exon(_StableRegion):
     
     def __repr__(self):
         my_type = self.__class__.__name__
-        return '%s(StableId=%s, Rank=%s)' % (my_type, self.StableId,self.Rank)
+        return '%s(StableId=%s, Rank=%s)' % (my_type, self.StableId, self.Rank)
     
     def __cmp__(self, other):
         return cmp(self.Rank, other.Rank)
@@ -779,6 +831,31 @@ class Exon(_StableRegion):
     Symbol = property(_get_symbol)
     
 
+
+class Intron(GenericRegion):
+    Type = 'intron'
+    def __init__(self, genome, db, rank, transcript_stable_id, Location=None):
+        GenericRegion.__init__(self, genome, db, Location=Location)
+        self.TranscriptStableId = transcript_stable_id
+        self.Rank = rank
+    
+    def __str__(self):
+        return self.__repr__()
+    
+    def __repr__(self):
+        my_type = self.__class__.__name__
+        return '%s(TranscriptId=%s, Rank=%s)' % (my_type,
+                            self.TranscriptStableId, self.Rank)
+    
+    def _make_symbol(self):
+        self._cached['Symbol'] = '%s-%s'%(self.TranscriptStableId, self.Rank)
+    
+    def _get_symbol(self):
+        return self._get_cached_value('Symbol', self._make_symbol)
+    
+    Symbol = property(_get_symbol)
+
+
 class Est(Gene):
     """an EST region"""
     Type = 'est'
@@ -802,7 +879,8 @@ class Variation(_Region):
                                 FlankingSeq='flanking_sequence',
                                 PeptideAlleles='transcript_variation',
                                 TranslationLocation='transcript_variation',
-                                Location='variation_feature')
+                                Location='variation_feature',
+                                AlleleFreqs='allele')
     
     def __init__(self, genome, db=None, Effect=None, Symbol=None, data=None):
         self.genome = genome
@@ -811,6 +889,7 @@ class Variation(_Region):
         self.variation_feature_table = get_table('variation_feature')
         self.transcript_variation_table = get_table('transcript_variation')
         self.flanking_sequence_table = get_table('flanking_sequence')
+        self.allele_table = get_table('allele')
         
         super(Variation, self).__init__()
         
@@ -901,6 +980,28 @@ class Variation(_Region):
     
     Alleles = property(_get_alleles)
     
+    def _get_allele_table_record(self):
+        variation_id = self._table_rows['variation_feature']['variation_id']
+        allele_table = self.allele_table
+        query = sql.select([allele_table],
+                allele_table.c.variation_id == variation_id)
+        records = [r for r in query.execute()]
+        
+        if len(records) == 0:
+            self._cached[('AlleleFreqs')] = self.NULL_VALUE
+            return
+        
+        self._table_rows['allele_table'] = records
+        data = [(rec['allele'], rec['frequency'], rec['sample_id']) for rec in records]
+        table = Table(header='allele freq sample_id'.split(), rows=data)
+        self._cached[('AlleleFreqs')] = table.sorted(['sample_id', 'allele'])
+    
+    def _get_allele_freqs(self):
+        return self._get_cached_value('AlleleFreqs',
+                        self._get_allele_table_record)
+    
+    AlleleFreqs = property(_get_allele_freqs)
+    
     def _get_symbol(self):
         return self._get_cached_value('Symbol',
                         self._get_variation_table_record)
