@@ -42,19 +42,17 @@ from cogent.evolve.substitution_calculation import (
     PartitionDefn, NonParamDefn, AlignmentAdaptDefn, ExpDefn, 
     ConstDefn, GammaDefn, MonotonicDefn, SelectForDimension, 
     WeightedPartitionDefn, chooseFastExponentiators)
+from cogent.evolve.discrete_markov import PsubMatrixDefn
 from cogent.evolve.likelihood_tree import makeLikelihoodTreeLeaf
 from cogent.maths.optimiser import ParameterOutOfBoundsError
 
-import logging
-LOG = logging.getLogger('cogent')
-
-__author__ = "Gavin Huttley and Andrew Butterfield"
+__author__ = "Peter Maxwell, Gavin Huttley and Andrew Butterfield"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __contributors__ = ["Gavin Huttley", "Andrew Butterfield", "Peter Maxwell",
                     "Matthew Wakefield", "Brett Easton", "Rob Knight",
                     "Von Bing Yap"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -166,7 +164,7 @@ class _SubstitutionModel(object):
         # ALPHABET        
         if recode_gaps:
             if model_gaps:
-                LOG.warning("Converting gaps to wildcards AND modeling gaps")
+                warnings.warn("Converting gaps to wildcards AND modeling gaps")
             else:
                 model_gaps = False
         
@@ -355,6 +353,39 @@ class _SubstitutionModel(object):
         return defns
 
 
+class DiscreteSubstitutionModel(_SubstitutionModel):
+    _default_expm_setting = None
+    
+    def _isInstantaneous(self, x, y):
+        return True
+    
+    def getParamList(self):
+        return []
+        
+    def makePsubsDefn(self, **kw):
+        motifs = tuple(self.getAlphabet())
+        psub = PsubMatrixDefn(
+            name="psubs", dimension = ('motif', motifs), default=None, 
+            dimensions=('locus', 'edge'))
+        return psub
+        
+    def makeFundamentalParamControllerDefns(self, bin_names):                
+        (input_probs, word_probs, mprobs_matrix) = \
+                self.mprob_model.makeMotifWordProbDefns()
+        
+        # XXX absorb 1 unneeded input
+        word_probs = CalcDefn(
+                lambda x,y:x, name='hack')(
+                word_probs, mprobs_matrix)
+        
+        defns = {
+            'motif_probs': input_probs,  
+            'word_probs': word_probs,
+            'bprobs': None,
+            }
+        return defns
+
+
 class _ContinuousSubstitutionModel(_SubstitutionModel):
     # subclass must provide:
     #
@@ -712,7 +743,7 @@ class SubstitutionModel(_ContinuousSubstitutionModel):
             if redundancyInPredicateMasks(predicate_masks):
                 raise ValueError("Redundancy in predicates.")
             if redundancyInPredicateMasks(predicates_plus_scale):
-                LOG.warning("do_scaling=True would be more efficient than"
+                warnings.warn("do_scaling=True would be more efficient than"
                         " these overly general predicates")
 
         self.predicate_masks = predicate_masks
