@@ -1,21 +1,25 @@
 #!/usr/bin/env python
 import os, time, cPickle
+from cogent.util import parallel
 
 __author__ = "Peter Maxwell"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Peter Maxwell"
 __email__ = "pm67nz@gmail.com"
 __status__ = "Production"
 
 class Checkpointer(object):
-    def __init__(self, filename, interval=600, noisy=True):
+    def __init__(self, filename, interval=None, noisy=True):
+        if interval is None:
+            interval = 1800
         self.filename = filename
         self.interval = interval
         self.last_time = time.time()
         self.noisy = noisy
+        self._redundant = parallel.getCommunicator().Get_rank() > 0
     
     def available(self):
         return self.filename is not None and os.path.exists(self.filename)
@@ -29,7 +33,7 @@ class Checkpointer(object):
         return obj
     
     def record(self, obj, msg=None, always=False):
-        if self.filename is None:
+        if self.filename is None or self._redundant:
             return
         now = time.time()
         elapsed = now - self.last_time
