@@ -3,28 +3,44 @@
 """
 
 import types
+import sys
 from time import clock
+from optparse import OptionParser, OptionGroup
 from datetime import datetime
 from string import maketrans, strip
 from random import randrange, choice
 from sys import maxint
-from os import popen, remove, makedirs
-from os.path import join, abspath
+from os import popen, remove, makedirs, getenv
+from os.path import join, abspath, exists, isdir
 from numpy import logical_not, sum
 from cPickle import dumps, loads
 from gzip import GzipFile
-
+import hashlib
 __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Amanda Birmingham",
                     "Sandra Smit", "Zongzhi Liu", "Daniel McDonald",
                     "Kyle Bittinger", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
 
+def safe_md5(open_file, block_size=2**20):
+    """Computes an md5 sum without loading the file into memory
+
+    This method is based on the answers given in:
+    http://stackoverflow.com/questions/1131220/get-md5-hash-of-a-files-without-open-it-in-python
+    """
+    md5 = hashlib.md5()
+    data = True
+    while data:
+        data = open_file.read(block_size)
+        if data:
+            md5.update(data)
+    return md5
+
 def identity(x):
     """Identity function: useful for avoiding special handling for None."""
     return x
@@ -1056,11 +1072,11 @@ class MappedDict(ConstrainedDict):
         """Ensure that has_key applies the mask."""
         return super(MappedDict, self).has_key(self.Mask(item))
 
-def getNewId():
+def getNewId(rand_f=randrange):
     """Creates a random 12-digit integer to be used as an id."""
 
     NUM_DIGITS = 12
-    return ''.join(map(str, [randrange(10) for i in range(NUM_DIGITS)]))
+    return ''.join(map(str, [rand_f(10) for i in range(NUM_DIGITS)]))
 #end function getNewId
 
 def generateCombinations(alphabet, combination_length):
@@ -1166,7 +1182,7 @@ def makeNonnegInt(n):
     return n
 #end makeNonnegInt
 
-def revComp(seq, use_DNA=True):
+def reverse_complement(seq, use_DNA=True):
     """Public function to reverse complement DNA or RNA sequence string
     
     seq: a string
@@ -1190,6 +1206,10 @@ def revComp(seq, use_DNA=True):
 
     #join the reverse-complemented list and return
     return "".join(comp_list)
+# The reverse_complement function was previously called revComp, but that 
+# naming doesn't adhere to the PyCogent coding guidelines. Renamed, but 
+# keeping the old name around to not break existing code.
+revComp = reverse_complement
 #end revComp
 
 def timeLimitReached(start_time, time_limit):
@@ -1277,18 +1297,104 @@ def NestedSplitter(delimiters=[None], same_level=False,
     return parser
 #end NestedSplitter
 
-def app_path(app):
-    """Returns path to an app, or empty string if no path.
-
-    Should generalize to work on Windows? No.
+def app_path(app,env_variable='PATH'):
+    """Returns path to an app, or False if app does not exist in env_variable
+    
+     This functions in the same way as which in that it returns
+     the first path that contains the app.
+    
     """
-    # redirect stderr to avoid junk that isn't collected by popen
-    cmd = ' '.join(['which', app, '2> /dev/null'])
-    result = popen(cmd).read().strip()
-    # Ends with 'not found' fixes bug in OSX 10.4
-    if not result or result.startswith('no') or result.endswith('not found'):
-        return False
-    return result
+    # strip off " characters, in case we got a FilePath object
+    app = app.strip('"')
+    paths = getenv(env_variable).split(':')
+    for path in paths:
+        p = join(path,app)
+        if exists(p):
+            return p
+    return False
+
+#some error codes for creating a dir
+def get_create_dir_error_codes():
+    return {'NO_ERROR':      0,
+            'DIR_EXISTS':    1,
+            'FILE_EXISTS':   2,
+            'OTHER_OS_ERROR':3}
+
+def create_dir(dir_name, fail_on_exist=False, handle_errors_externally=False):
+    """Create a dir safely and fail meaningful.
+
+    dir_name: name of directory to create
+
+    fail_on_exist: if true raise an error if dir already exists
+    
+    handle_errors_externally: if True do not raise Errors, but return
+                   failure codes. This allows to handle errors locally and
+                   e.g. hint the user at a --force_overwrite options.
+                   
+    returns values (if no Error raised):
+    
+         0:  dir could be safely made
+         1:  directory already existed
+         2:  a file with the same name exists          
+         3:  any other unspecified OSError
+
+
+    See qiime/denoiser.py for an example of how to use this mechanism.
+
+    Note: Depending  of how thorough we want to be we could add tests,
+          e.g. for testing actual write permission in an existing dir.
+    """
+    error_code_lookup = get_create_dir_error_codes()
+    #pre-instanciate function with
+    ror = curry(handle_error_codes, dir_name, handle_errors_externally)
+
+    if exists(dir_name):
+        if isdir(dir_name):
+            #dir is there
+            if fail_on_exist:
+                return ror(error_code_lookup['DIR_EXISTS'])
+            else:
+                return error_code_lookup['DIR_EXISTS']
+        else:
+            #must be file with same name
+            return ror(error_code_lookup['FILE_EXISTS'])
+    else:
+        #no dir there, try making it
+        try:
+            makedirs(dir_name)
+        except OSError:
+            return ror(error_code_lookup['OTHER_OS_ERROR'])
+    
+    return error_code_lookup['NO_ERROR']
+
+def handle_error_codes(dir_name, supress_errors=False,
+                       error_code=None):
+    """Wrapper function for error_handling.
+
+    dir_name: name of directory that raised the error
+    suppress_errors: if True raise Errors, otherwise return error_code
+    error_code: the code for the error
+    """
+    error_code_lookup = get_create_dir_error_codes()
+    
+    if error_code == None:
+        error_code = error_code_lookup['NO_ERROR']
+    
+    error_strings = \
+        {error_code_lookup['DIR_EXISTS'] :
+          "Directory already exists: %s" % dir_name,
+         error_code_lookup['FILE_EXISTS'] : 
+          "File with same name exists: %s" % dir_name,
+         error_code_lookup['OTHER_OS_ERROR']: 
+          "Could not create output directory: %s. " % dir_name +
+          "Check the permissions."}
+
+    if error_code == error_code_lookup['NO_ERROR']:
+        return error_code_lookup['NO_ERROR']
+    if supress_errors:
+        return error_code
+    else:
+        raise OSError, error_strings[error_code]
 
 def remove_files(list_of_filepaths, error_on_missing=True):
     """Remove list of filepaths, optionally raising an error if any are missing
@@ -1350,3 +1456,167 @@ def get_random_directory_name(suppress_mkdir=False,\
     if return_absolute_path:
         return abs_dirpath
     return dirpath
+
+def build_usage_lines(required_options,
+    script_description,
+    script_usage,
+    optional_input_line,
+    required_input_line):
+    """ Build the usage string from components 
+    """
+    line1 = 'usage: %prog [options] ' + '{%s}' %\
+     ' '.join(['%s %s' % (str(ro),ro.dest.upper())\
+               for ro in required_options])
+    usage_examples = []
+    for title, description, command in script_usage:
+        title = title.strip(':').strip()
+        description = description.strip(':').strip()
+        command = command.strip()
+        if title:
+            usage_examples.append('%s: %s\n %s' %\
+             (title,description,command))
+        else:
+            usage_examples.append('%s\n %s' % (description,command))
+    usage_examples = '\n\n'.join(usage_examples)
+    lines = (line1,
+             '', # Blank line
+             optional_input_line,
+             required_input_line,
+             '', # Blank line
+             script_description,
+             '', # Blank line
+             'Example usage: ',\
+             'Print help message and exit',
+             ' %prog -h\n',
+             usage_examples)
+    return '\n'.join(lines)
+
+## Begin functions for handling command line parameters
+
+def set_parameter(key,kwargs,default=None):
+    try:
+        return kwargs[key]
+    except KeyError:
+        return default
+        
+def set_required_parameter(key,kwargs):
+    try:
+        return kwargs[key]
+    except KeyError:
+        raise KeyError,\
+         "parse_command_line_parameters requires value for %s" % key
+        
+def parse_command_line_parameters(**kwargs):
+    """ Constructs the OptionParser object and parses command line arguments
+    
+        parse_command_line_parameters takes a dict of objects via kwargs which
+         it uses to build command line interfaces according to standards 
+         developed in the Knight Lab, and enforced in QIIME. The currently 
+         supported options are listed below with their default values. If no 
+         default is provided, the option is required.
+        
+        script_description
+        script_usage
+        version
+        required_options=None
+        optional_options=None
+        suppress_verbose=False
+        disallow_positional_arguments=True
+        help_on_no_arguments=True
+        optional_input_line = '[] indicates optional input (order unimportant)'
+        required_input_line = '{} indicates required input (order unimportant)'
+        
+       These values can either be passed directly, as:
+        parse_command_line_parameters(script_description="My script",\
+                                     script_usage=[('Print help','%prog -h','')],\
+                                     version=1.0)
+                                     
+       or they can be passed via a pre-constructed dict, as:
+        d = {'script_description':"My script",\
+             'script_usage':[('Print help','%prog -h','')],\
+             'version':1.0}
+        parse_command_line_parameters(**d)
+    
+    """
+    # Get the options, or defaults if none were provided.
+    script_description = set_required_parameter('script_description',kwargs)
+    script_usage = set_required_parameter('script_usage',kwargs)
+    version = set_required_parameter('version',kwargs)
+    required_options = set_parameter('required_options',kwargs,[])
+    optional_options = set_parameter('optional_options',kwargs,[])
+    suppress_verbose = set_parameter('suppress_verbose',kwargs,False)
+    disallow_positional_arguments =\
+     set_parameter('disallow_positional_arguments',kwargs,True)
+    help_on_no_arguments = set_parameter('help_on_no_arguments',kwargs,True)
+    optional_input_line = set_parameter('optional_input_line',kwargs,\
+        '[] indicates optional input (order unimportant)')
+    required_input_line = set_parameter('required_input_line',kwargs,\
+        '{} indicates required input (order unimportant)')
+    # command_line_text will usually be nothing, but can be passed for
+    # testing purposes
+    command_line_args = set_parameter('command_line_args',kwargs,None)
+    
+    # Build the usage and version strings
+    usage = build_usage_lines(required_options,script_description,script_usage,\
+                              optional_input_line,required_input_line)
+    version = 'Version: %prog ' + version
+    
+    # Instantiate the command line parser object
+    parser = OptionParser(usage=usage, version=version)
+    
+    # If no arguments were provided, print the help string (unless the
+    # caller specified not to)
+    if help_on_no_arguments and len(sys.argv) == 1:
+        parser.print_usage()
+        parser.exit()
+    
+    # Process the required options
+    if required_options:
+        # Define an option group so all required options are
+        # grouped together, and under a common header
+        required = OptionGroup(parser, "REQUIRED options",
+         "The following options must be provided under all circumstances.")
+        for ro in required_options:
+            # if the option doesn't already end with [REQUIRED], 
+            # add it.
+            if not ro.help.strip().endswith('[REQUIRED]'):
+                ro.help += ' [REQUIRED]'
+            required.add_option(ro)
+        parser.add_option_group(required)
+
+    # Add a verbose parameter (if the caller didn't specify not to)
+    if not suppress_verbose:
+        parser.add_option('-v','--verbose',action='store_true',\
+           dest='verbose',help='Print information during execution '+\
+           '-- useful for debugging [default: %default]',default=False)
+
+    # Add the optional options
+    map(parser.add_option,optional_options)
+    
+    # Parse the command line
+    # command_line_text will None except in test cases, in which 
+    # case sys.argv[1:] will be parsed
+    opts,args = parser.parse_args(command_line_args)
+    
+    # If positional arguments are not allowed, and any were provided,
+    # raise an error.
+    if disallow_positional_arguments and len(args) != 0:
+        parser.error("Positional argument detected: %s\n" % str(args[0]) +\
+         " Be sure all parameters are identified by their option name.\n" +\
+         " (e.g.: include the '-i' in '-i INPUT_DIR')")
+
+    # Test that all required options were provided.
+    if required_options:
+        required_option_ids = [o.dest for o in required.option_list]
+        for required_option_id in required_option_ids:
+            if getattr(opts,required_option_id) == None:
+                parser.error('Required option --%s omitted.' \
+                             % required_option_id)
+            
+    # Return the parser, the options, and the arguments. The parser is returned
+    # so users have access to any additional functionality they may want at 
+    # this stage -- most commonly, it will be used for doing custom tests of 
+    # parameter values.
+    return parser, opts, args
+    
+## End functions for handling command line parameters
