@@ -18,7 +18,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Felix Schill"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -411,7 +411,7 @@ class Table(DictArray):
     
     def _callback(self, callback, row, columns=None, num_columns=None):
         if callable(callback):
-            row_segment = [row[col] for col in columns]
+            row_segment = row.take(columns)
             if num_columns == 1:
                 row_segment = row_segment[0]
             return callback(row_segment)
@@ -439,10 +439,17 @@ class Table(DictArray):
             num_columns = None
         
         row_indexes = []
-        for rdex, row in enumerate(self):
-            if self._callback(callback, row, columns, num_columns):
+        if not callable(callback):
+            data = self
+            cols = columns
+        else:
+            data = self.array
+            cols = map(self.Header.index, columns)
+        
+        for rdex, row in enumerate(data):
+            if self._callback(callback, row, cols, num_columns):
                 row_indexes.append(rdex)
-                
+            
         sub_set = numpy.take(self, row_indexes, 0)
         
         kw = self._get_persistent_attrs()
@@ -486,8 +493,15 @@ class Table(DictArray):
             num_columns = None
         
         count = 0
-        for row in self:
-            if self._callback(callback, row, columns, num_columns):
+        if not callable(callback):
+            data = self
+            cols = columns
+        else:
+            data = self.array
+            cols = map(self.Header.index, columns)
+        
+        for row in data:
+            if self._callback(callback, row, cols, num_columns):
                 count += 1
         return count
     
@@ -507,32 +521,44 @@ class Table(DictArray):
         elif isinstance(columns, str):
             columns = [columns]
         
+        indices = [self.Header.index(col) for col in columns]
+        
         if not reverse:
             is_reversed = [False] * len(columns)
+            reverse_indices = []
         else:
+            if type(reverse) == str:
+                reverse = [reverse]
+            reverse_indices = []
+            for index, header_index in enumerate(indices):
+                col = self.Header[header_index]
+                if col in reverse:
+                    reverse_indices += [index]
+            
             is_reversed = [col in reverse for col in columns]
         
-        indices = zip([self.Header.index(col) for col in columns],
-                       is_reversed)
+        reverse_indices = numpy.array(reverse_indices)
+        
+        dtypes = [(self.Header[i], self.array.dtype) for i in indices]
         
         # applying the decorate-sort-undecorate approach
-        aux_list = []
-        for row in self:
-            new_row = []
-            for index, reverse in indices:
-                if reverse is False:
-                    new_row.append(row[index])
-                    continue
-                
-                try:
-                    new_row.append(-row[index])
-                except TypeError:
-                    new_row.append(row[index].translate(_reversed_chrs))
-            aux_list.append((new_row, row))
+        aux_list = self.array.take(indices, axis=1)
         
-        aux_list.sort()
+        # we figure out the casting funcs for any reversed elements
+        cast = []
+        for index in reverse_indices:
+            val = aux_list[0, index]
+            try:
+                val = val.translate(_reversed_chrs)
+                func = lambda x: x.translate(_reversed_chrs)
+            except AttributeError:
+                func = lambda x: x * -1
+            func = numpy.vectorize(func)
+            aux_list[:, index] = func(aux_list[:, index])
         
-        new_twoD = [list(row[-1]) for row in aux_list]
+        aux_list = numpy.rec.fromarrays(aux_list.copy().T, dtype=dtypes)
+        indices = aux_list.argsort()
+        new_twoD = self.array.take(indices, axis=0)
         
         kw = self._get_persistent_attrs()
         kw.update(kwargs)
@@ -606,8 +632,15 @@ class Table(DictArray):
         else:
             num_columns = None
         
-        twoD = [list(row) + [self._callback(callback, row, columns,
-                num_columns)] for row in self]
+        if not callable(callback):
+            data = self
+            cols = columns
+        else:
+            data = self.array
+            cols = map(self.Header.index, columns)
+        
+        twoD = [list(row) + [self._callback(callback, row, cols,
+                num_columns)] for row in data]
         
         kw = self._get_persistent_attrs()
         kw.update(kwargs)
