@@ -1,11 +1,11 @@
 #!/usr/bin/env python
-__all__ = ['test_goodness_of_fit', 'test_UPGMA', 'test_metric_scaling', 'test_procrustes', 'test_nmds']
+__all__ = ['test_goodness_of_fit', 'test_UPGMA', 'test_metric_scaling', 'test_procrustes', 'test_nmds', 'test_approximate_mds']
 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Catherine Lozuopone", "Peter Maxwell", "Rob Knight", "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
