@@ -10,7 +10,7 @@ __author__ = "Peter Maxwell, Gavin Huttley and Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -153,13 +153,34 @@ class AlignmentTestMethods(unittest.TestCase):
     def test_slidingWindows(self):          
         """test slicing of sequences"""      
         alignment = LoadSeqs(data = {'seq1': 'ACGTACGT', 'seq2': 'ACGTACGT', 'seq3': 'ACGTACGT'})
-        result = []                          
+        result = []
         for bit in alignment.slidingWindows(5,2):
-            result+=[bit]                    
+            result+=[bit]
         self.assertEqual(result[0].todict(), {'seq3': 'ACGTA', 'seq2': 'ACGTA', 'seq1': 'ACGTA'})
         self.assertEqual(result[1].todict(), {'seq3': 'GTACG', 'seq2': 'GTACG', 'seq1': 'GTACG'})
-                                             
-        result = []                          
+        
+        # specify a starting window position
+        result = []
+        for bit in alignment.slidingWindows(5,2, start=1):
+            result+=[bit]
+        self.assertEqual(result[0].todict(), {'seq3': 'CGTAC', 'seq2': 'CGTAC', 'seq1': 'CGTAC'})
+        self.assertEqual(result[1].todict(), {'seq3': 'TACGT', 'seq2': 'TACGT', 'seq1': 'TACGT'})
+        
+        # specify a ending window position
+        result = []
+        for bit in alignment.slidingWindows(5, 1, start=1, end=3):
+            result+=[bit]
+        self.assertEqual(result[0].todict(), {'seq3': 'CGTAC', 'seq2': 'CGTAC', 'seq1': 'CGTAC'})
+        self.assertEqual(result[1].todict(), {'seq3': 'GTACG', 'seq2': 'GTACG', 'seq1': 'GTACG'})
+        
+        # start conditions < window-size from end don't return a window
+        # specify a ending window position
+        result = []
+        for bit in alignment.slidingWindows(5, 1, start=5):
+            result+=[bit]
+        self.assertEqual(result, [])
+        
+        result = []
         for bit in alignment.slidingWindows(5,1):
             result+=[bit]                    
         self.assertEqual(result[0].todict(), {'seq3': 'ACGTA', 'seq2': 'ACGTA', 'seq1': 'ACGTA'})
@@ -446,6 +467,18 @@ class SequenceTestMethods(unittest.TestCase):
                           'GTAGC', 'TAGCA', 'AGCAT', 'GCATA', 'CATAG',
                           'ATAGC', 'TAGCT', 'AGCTC', 'GCTCG', 'CTCGA'])
     
+        result = []
+        for bit in self.seq.slidingWindows(5, 1, start=3, end=6):
+            result+=[bit]
+        self.assertEqual([str(x) for x in result],
+                         ['ACGTT', 'CGTTG', 'GTTGC'])
+        
+        # should not get a window when starting conditions don't generate one
+        result = []
+        for bit in self.seq.slidingWindows(20, 1, start=6):
+            result += [bit]
+        self.assertEqual(result, [])
+    
     def test_reversecomplement(self):
         """testing reversal and complementing of a sequence"""
         seq = Sequence(DNA, seq='ACTGTAA')
