@@ -6,17 +6,18 @@ from cogent.db.ensembl.host import HostAccount, get_ensembl_account
 from cogent.db.ensembl.util import convert_strand
 from cogent.db.ensembl.genome import Genome
 from cogent.db.ensembl.sequence import _assemble_seq
+from cogent.db.ensembl.util import asserted_one
 
 __author__ = "Gavin Huttley, Hua Ying"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "hua Ying"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
 
-Release = 56
+Release = 58
 
 if 'ENSEMBL_ACCOUNT' in os.environ:
     username, password = os.environ['ENSEMBL_ACCOUNT'].split()
@@ -29,16 +30,16 @@ class GenomeTestBase(TestCase):
     mouse = Genome(Species="mouse", Release=Release, account=account)
     rat = Genome(Species="rat", Release=Release, account=account)
     macaq = Genome(Species="macaque", Release=Release, account=account)
-    brca2 = list(human.getGenesMatching(StableId="ENSG00000139618"))[0]
+    brca2 = human.getGeneByStableId(StableId="ENSG00000139618")
 
 class TestGenome(GenomeTestBase):
     
     def test_other_features(self):
-        """should correctly return record for ENSESTG00000000020"""
-        est = self.human.getEstMatching(StableId='ENSESTG00000000020')
+        """should correctly return record for ENSESTG00007281025"""
+        est = self.human.getEstMatching(StableId='ENSESTG00007281025')
         direct = list(est)[0]
         ests = self.human.getFeatures(feature_types='est', CoordName=8,
-                                                Start=105711700, End=105715900)
+                                                Start=145164000, End=145166000)
         stable_ids = [est.StableId for est in ests]
         self.assertContains(stable_ids, direct.StableId)
     
@@ -99,7 +100,13 @@ class TestGenome(GenomeTestBase):
         """excercising ability to specify pool connection"""
         dog = Genome(Species="dog", Release=Release, account=account,
                 pool_recycle=1000)
-
+    
+    def test_gorilla(self):
+        """should correctly return a gorilla gene"""
+        gorilla = Genome(Species="gorilla", Release=Release, account=account)
+        gene = gorilla.getGeneByStableId('ENSGGOG00000005730')
+        self.assertEquals(str(gene.Seq[:10]), 'TGGGAGTCCA')
+    
 
 class TestGene(GenomeTestBase):
     def _eval_brca2(self, brca2):
@@ -131,13 +138,11 @@ class TestGene(GenomeTestBase):
     def test_translated_exons(self):
         """should correctly translate a gene with 2 exons but 1st exon
         transcribed"""
-        gene = self.mouse.getGenesMatching(StableId='ENSMUSG00000036136')
-        gene = list(gene)[0]
+        gene = self.mouse.getGeneByStableId(StableId='ENSMUSG00000036136')
         transcript = gene.getMember('ENSMUST00000041133')
         self.assertTrue(len(transcript.ProteinSeq) > 0)
         # now one on the - strand
-        gene = self.mouse.getGenesMatching(StableId='ENSMUSG00000045912')
-        gene = list(gene)[0]
+        gene = self.mouse.getGeneByStableId(StableId='ENSMUSG00000045912')
         transcript = gene.Transcripts[0]
         self.assertTrue(len(transcript.ProteinSeq) > 0)
     
@@ -147,8 +152,7 @@ class TestGene(GenomeTestBase):
         # always right. This case has a macaque gene which we correctly
         # infer the CDS boundaries for according to Ensembl, but the CDS
         # length is not divisible by 3, hence our code fails.
-        gene = self.macaq.getGenesMatching(StableId='ENSMMUG00000001551')
-        gene = list(gene)[0]
+        gene = self.macaq.getGeneByStableId(StableId='ENSMMUG00000001551')
         transcript = gene.getMember('ENSMMUT00000002194')
         try:
             # this fails because of the above length issue
@@ -167,7 +171,7 @@ class TestGene(GenomeTestBase):
     def test_gene_transcripts(self):
         """should return multiple transcripts"""
         stable_id = 'ENSG00000012048'
-        gene = list(self.human.getGenesMatching(StableId=stable_id))[0]
+        gene = self.human.getGeneByStableId(StableId=stable_id)
         self.assertTrue(len(gene.Transcripts) > 1)
         # .. and correctly construct the Cds and location
         for transcript in gene.Transcripts:
@@ -177,14 +181,13 @@ class TestGene(GenomeTestBase):
     def test_get_longest_cds_transcript(self):
         """should correctly return transcript with longest cds"""
         stable_id = 'ENSG00000178591'
-        gene = list(self.human.getGenesMatching(StableId=stable_id))[0]
+        gene = self.human.getGeneByStableId(StableId=stable_id)
         ts = gene.getLongestCdsTranscript()
         self.assertEquals(ts.getCdsLength(), max(gene.getCdsLengths()))
     
     def test_rna_transcript_cds(self):
         """should return a Cds for an RNA gene too"""
-        rna_gene = self.human.getGenesMatching(StableId='ENSG00000210049')
-        rna_gene = list(rna_gene)[0]
+        rna_gene = self.human.getGeneByStableId(StableId='ENSG00000210049')
         self.assertTrue(rna_gene.Transcripts[0].getCdsLength() > 0)
     
     def test_gene_annotation(self):
@@ -203,7 +206,7 @@ class TestGene(GenomeTestBase):
     def test_get_by_description(self):
         """if get by description, all attributes should be correctly
         constructed"""
-        description='breast cancer type 2'
+        description='breast cancer 2'
         results = list(self.human.getGenesMatching(Description=description))
         self.assertEquals(len(results), 1)
         self._eval_brca2(results[0])
@@ -232,12 +235,91 @@ class TestGene(GenomeTestBase):
         """combining the description and biotype should return a result"""
         results = list(self.human.getGenesMatching(BioType="protein_coding",
                     Description="cancer"))
-        self.assertTrue(len(results) > 100)
+        self.assertTrue(len(results) > 50)
     
     def test_variant(self):
         """variant attribute correctly constructed"""
         self.assertTrue(len(self.brca2.Variants) > 880)
     
+    def test_get_gene_by_stable_id(self):
+        """should correctly handle getting gene by stable_id"""
+        stable_id = 'ENSG00000012048'
+        gene = self.human.getGeneByStableId(StableId=stable_id)
+        self.assertEquals(gene.StableId, stable_id)
+        
+        # if invalid stable_id, should just return None
+        stable_id = 'ENSG00000XXXXX'
+        gene = self.human.getGeneByStableId(StableId=stable_id)
+        self.assertEquals(gene, None)
+    
+    def test_intron_number(self):
+        """number of introns should be correct"""
+        for gene_id, transcript_id, exp_number in [
+                            ('ENSG00000234914', 'ENST00000434799', 0), 
+                            ('ENSG00000132199', 'ENST00000319815', 14), 
+                            ('ENSG00000132199', 'ENST00000383578', 15)]:
+            gene = asserted_one(self.human.getGenesMatching(StableId=gene_id))
+            transcript = asserted_one(
+                [t for t in gene.Transcripts if t.StableId==transcript_id])
+            if exp_number == 0:
+                self.assertEqual(transcript.Introns, None)
+            else:
+                self.assertEqual(len(transcript.Introns), exp_number)
+        
+    
+    def test_intron(self):
+        """should get correct Intron sequence, regardless of strand"""
+        # IL2 is on - strand, IL13 is on + strand, both have three introns
+        IL2_exp_introns = [
+                    (1, 123377358, 123377448, 'gtaagtatat', 'actttcttag'),
+                    (2, 123375008, 123377298, 'gtaagtacaa', 'attattctag'),
+                    (3, 123373017,123374864, 'gtaaggcatt', 'tcttttatag')]
+        IL13_exp_introns = [
+                    (1, 131994052, 131995109, 'gtgagtgtcg', 'gctcccacag'),
+                    (2, 131995163, 131995415, 'gtaaggacct', 'ctccccacag'),
+                    (3, 131995520, 131995866, 'gtaaggcatc', 'tgtcctgcag')]
+        
+        for symbol, stable_id, exp_introns in [
+                    ('IL2', 'ENST00000226730', IL2_exp_introns), 
+                    ('IL13', 'ENST00000304506', IL13_exp_introns)]:
+            gene = asserted_one(self.human.getGenesMatching(Symbol=symbol))
+            strand = gene.Location.Strand
+            transcript = asserted_one(
+                [t for t in gene.Transcripts if t.StableId==stable_id])
+            introns = transcript.Introns
+            self.assertEqual(len(introns), len(exp_introns))
+            idx = 0
+            for intron in introns:
+                loc = intron.Location
+                start, end = loc.Start, loc.End
+                seq = str(intron.Seq)
+                exp_rank, exp_start, exp_end, exp_seq5, \
+                                    exp_seq3 = exp_introns[idx]
+                self.assertEqual(loc.Strand, strand)
+                # test the order using rank
+                self.assertEqual(intron.Rank, exp_rank)
+                # test position 
+                self.assertEqual(start, exp_start)
+                self.assertEqual(end, exp_end)
+                # test sequence
+                self.assertEqual(seq[:10], exp_seq5.upper())
+                self.assertEqual(seq[-10:], exp_seq3.upper())
+                idx += 1
+    
+    def test_intron_annotation(self):
+        """sequences annotated with Introns should return correct seq"""
+        for symbol, stable_id, rank, exp_seq5, exp_seq3 in [
+                ('IL2', 'ENST00000226730', 1, 'gtaagtatat', 'actttcttag'), 
+                ('IL13', 'ENST00000304506', 3, 'gtaaggcatc', 'tgtcctgcag')]:
+            gene = asserted_one(self.human.getGenesMatching(Symbol=symbol))
+            seq = gene.getAnnotatedSeq(feature_types='gene')
+            intron = asserted_one(seq.getAnnotationsMatching('intron',
+                                                '%s-%d'%(stable_id, rank)))
+            intron_seq = str(seq.getRegionCoveringAll(intron).getSlice())
+            self.assertEqual(intron_seq[:10], exp_seq5.upper())
+            self.assertEqual(intron_seq[-10:], exp_seq3.upper())
+        
+    
 
 class TestVariation(GenomeTestBase):
     snp_names =  ['rs34213141', 'rs12791610', 'rs10792769', 'rs11545807', 'rs11270496']
@@ -313,12 +395,19 @@ class TestVariation(GenomeTestBase):
         for symbol, status in snp_status:
             snp = list(self.human.getVariation(Symbol=symbol, validated=True))
             self.assertEquals(snp!=[], status)
-    
+        
+    def test_allele_freqs(self):
+        """exercising getting AlleleFreq data"""
+        snp = list(self.human.getVariation(Symbol='rs34213141'))[0]
+        expect = dict(A=0.030303, G=0.969697)
+        allele_freqs = snp.AlleleFreqs.getRawData(['allele', 'freq'])
+        for allele, freq in allele_freqs:
+            self.assertFloatEqual(freq, expect[allele], eps=1e-3)
+        
 
 class TestFeatures(GenomeTestBase):
     def setUp(self):
-        self.igf2 = list(
-                  self.human.getGenesMatching(StableId='ENSG00000167244'))[0]
+        self.igf2 = self.human.getGeneByStableId(StableId='ENSG00000167244')
     
     def test_CpG_island(self):
         """should return correct CpG islands"""
