@@ -22,6 +22,7 @@ from cogent.evolve import substitution_model, predicate
 from cogent import DNA, LoadSeqs, LoadTree
 from cogent.util.unit_test import TestCase, main
 from cogent.maths.matrix_exponentiation import PadeExponentiator as expm
+from cogent.maths.stats.information_criteria import aic, bic
 from cogent.evolve.models import JTT92
 
 Nucleotide = substitution_model.Nucleotide
@@ -32,7 +33,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight",
                     "Matthew Wakefield", "Brett Easton"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -108,12 +109,16 @@ class LikelihoodCalcs(TestCase):
     def setUp(self):
         self.alignment = ALIGNMENT.takeSeqs(OTU_NAMES)[0: 42]
         self.tree = LoadTree(tip_names=OTU_NAMES)
-        self.par_values = {'kappa': 3.0}
-        self.length = 1.0
     
-    def _makeLikelihoodFunction(self, submod, alignment, **kw):
+    def _makeLikelihoodFunction(self, submod, translate=False, **kw):
+        alignment = self.alignment
+        if translate:
+            alignment = alignment.getTranslation()
         calc = submod.makeLikelihoodFunction(self.tree, **kw)
         calc.setAlignment(alignment)
+        calc.setParamRule('length', value=1.0, is_const=True)
+        if not translate:
+            calc.setParamRule('kappa', value=3.0, is_const=True)
         return calc
     
     def test_no_seq_named_root(self):
@@ -139,14 +144,13 @@ class LikelihoodCalcs(TestCase):
             lf.setSequences(collection)
         except AssertionError:
             pass
-        
     
     def test_binned_gamma(self):
         """just rate is gamma distributed"""
         submod = substitution_model.Codon(
             predicates={'kappa': 'transition', 'omega': 'replacement'},
             ordered_param='rate', distribution='gamma', mprob_model='tuple')
-        lf = self._makeLikelihoodFunction(submod, self.alignment, bins=3)
+        lf = self._makeLikelihoodFunction(submod, bins=3)
         try:
             values = lf.getParamValueDict(['bin'])['omega_factor'].values()
         except KeyError:
@@ -164,7 +168,7 @@ class LikelihoodCalcs(TestCase):
             predicates={'kappa': 'transition', 'omega': 'replacement'},
             ordered_param='rate', partitioned_params='omega', 
             distribution='gamma', mprob_model='tuple')
-        lf = self._makeLikelihoodFunction(submod, self.alignment,bins=3) 
+        lf = self._makeLikelihoodFunction(submod,bins=3) 
         values = lf.getParamValueDict(['bin'])['omega_factor'].values()
         self.assertEqual(round(sum(values) / len(values), 6), 1.0)
         self.assertEqual(len(values), 3)
@@ -175,7 +179,7 @@ class LikelihoodCalcs(TestCase):
             predicates={'kappa': 'transition', 'omega': 'replacement'},
             ordered_param='rate', partitioned_params='omega', 
             distribution='free', mprob_model='tuple')
-        lf = self._makeLikelihoodFunction(submod, self.alignment, bins=3)
+        lf = self._makeLikelihoodFunction(submod, bins=3)
         values = lf.getParamValueDict(['bin'])['omega_factor'].values()
         self.assertEqual(round(sum(values) / len(values), 6), 1.0)
         self.assertEqual(len(values), 3)
@@ -185,7 +189,7 @@ class LikelihoodCalcs(TestCase):
             predicates={'kappa': 'transition', 'omega': 'replacement'},
             ordered_param='kappa', partitioned_params=['omega'], 
             mprob_model='tuple')
-        lf = self._makeLikelihoodFunction(submod, self.alignment,
+        lf = self._makeLikelihoodFunction(submod, 
                     bins=['slow', 'fast'])
         lf.setParamRule('kappa', value=1.0, is_const=True)
         lf.setParamRule('kappa', edge="Human", init=1.0, is_const=False)
@@ -202,15 +206,9 @@ class LikelihoodCalcs(TestCase):
             predicates={'kappa': 'transition', 'omega': 'replacement'},
             mprob_model='tuple')
         
-        self.par_values.update({'omega':0.5})
-        likelihood_function = self._makeLikelihoodFunction(
-                submod, self.alignment)
-                    
-        for par, val in self.par_values.items():
-            likelihood_function.setpar(par, val)
-            
-        likelihood_function.setpar("length", self.length)
-        evolve_lnL = likelihood_function.testfunction()
+        likelihood_function = self._makeLikelihoodFunction(submod)
+        likelihood_function.setParamRule('omega', value=0.5, is_const=True)
+        evolve_lnL = likelihood_function.getLogLikelihood()
         self.assertFloatEqual(evolve_lnL, -80.67069614541883)
     
     def test_nucleotide(self):
@@ -222,14 +220,25 @@ class LikelihoodCalcs(TestCase):
             predicates={'kappa': 'transition'})
         # now do using the evolve
         likelihood_function = self._makeLikelihoodFunction(
-                submod, self.alignment)
-        for par, val in self.par_values.items():
-            likelihood_function.setpar(par, val)
-            
-        likelihood_function.setpar("length", self.length)
-        evolve_lnL = likelihood_function.testfunction()
+                submod)
+        self.assertEqual(likelihood_function.getNumFreeParams(), 0)
+        evolve_lnL = likelihood_function.getLogLikelihood()
         self.assertFloatEqual(evolve_lnL, -157.49363874840455)
     
+    def test_discrete_nucleotide(self):
+        """test that partially discrete nucleotide model can be constructed, 
+        differs from continuous, and has the expected number of free params"""
+        submod = Nucleotide(
+            equal_motif_probs=True,
+            do_scaling=False,
+            motif_probs=None,
+            predicates={'kappa': 'transition'})
+        likelihood_function = self._makeLikelihoodFunction(
+                submod, discrete_edges=['Human'])
+        self.assertEqual(likelihood_function.getNumFreeParams(), 12)
+        evolve_lnL = likelihood_function.getLogLikelihood()
+        self.assertNotEqual(evolve_lnL,  -157.49363874840455)
+        
     def test_dinucleotide(self):
         """test a dinucleotide model."""
         submod = substitution_model.Dinucleotide(
@@ -238,25 +247,19 @@ class LikelihoodCalcs(TestCase):
                 motif_probs = None,
                 predicates = {'kappa': 'transition'},
                 mprob_model='tuple')
-        likelihood_function = self._makeLikelihoodFunction(
-                submod, self.alignment)
-        for par, val in self.par_values.items():
-            likelihood_function.setpar(par, val)
-            
-        likelihood_function.setpar("length", self.length)
-        evolve_lnL = likelihood_function.testfunction()
+        likelihood_function = self._makeLikelihoodFunction(submod)
+        evolve_lnL = likelihood_function.getLogLikelihood()
         self.assertFloatEqual(evolve_lnL, -102.48145536663735)
     
     def test_protein(self):
         """test a protein model."""
         submod = substitution_model.Protein(
             do_scaling=False, equal_motif_probs=True)
-        alignment = self.alignment.getTranslation()
         
-        likelihood_function = self._makeLikelihoodFunction(submod, alignment)
+        likelihood_function = self._makeLikelihoodFunction(submod, 
+                translate=True)
         
-        likelihood_function.setpar("length", self.length)
-        evolve_lnL = likelihood_function.testfunction()
+        evolve_lnL = likelihood_function.getLogLikelihood()
         self.assertFloatEqual(evolve_lnL, -89.830370754876185)
     
 
@@ -289,14 +292,28 @@ class LikelihoodFunctionTests(TestCase):
                 ("Human", 0.3),
                 ("HowlerMon", 0.4),
                 ("Mouse",  0.5)]:
-            likelihood_function.setpar("length", length, edge=species)
+            likelihood_function.setParamRule("length", value=length, 
+                    edge=species, is_const=True)
         for (species1, species2, length) in [
                 ("Human", "HowlerMon", 0.7),
                 ("Human", "Mouse", 0.6)]:
             LCA = self.tree.getConnectingNode(species1, species2).Name
-            likelihood_function.setpar("length", length, edge=LCA)
+            likelihood_function.setParamRule("length", value=length, 
+                    edge=LCA, is_const=True)
         
-        likelihood_function.setpar("beta", 4.0)
+        likelihood_function.setParamRule("beta", value=4.0, is_const=True)
+    
+    def test_information_criteria(self):
+        """test get information criteria from a model."""
+        lf = self._makeLikelihoodFunction()
+        nfp = lf.getNumFreeParams()
+        lnL = lf.getLogLikelihood()
+        l = len(self.data)
+        self.assertFloatEqual(lf.getAic(), aic(lnL, nfp))
+        self.assertFloatEqual(lf.getAic(second_order=True),
+            aic(lnL, nfp, l))
+        
+        self.assertFloatEqual(lf.getBic(), bic(lnL, nfp, l))
     
     def test_result_str(self):
         # actualy more a test of self._setLengthsAndBetas()
@@ -356,7 +373,7 @@ motif  mprobs
         likelihood_function = self._makeLikelihoodFunction()
         self._setLengthsAndBetas(likelihood_function)
         self.assertAlmostEquals(-250.686745262,
-            likelihood_function.testfunction(),places=9)
+            likelihood_function.getLogLikelihood(),places=9)
     
     def test_ancestralsequences(self):
         likelihood_function = self._makeLikelihoodFunction()
@@ -405,8 +422,8 @@ motif  mprobs
         al = LoadSeqs(data={'a':'ggaatt','c':'cctaat'})
         t = LoadTree(treestring="(a,c);")
         sm = substitution_model.Dinucleotide(mprob_model='tuple')
-        pc = sm.makeParamController(t)
-        lf = pc.makeCalculator(al)
+        lf = sm.makeParamController(t)
+        lf.setAlignment(al)
         simalign = lf.simulateAlignment()
         self.assertEqual(len(simalign), 6)
     
@@ -420,9 +437,9 @@ motif  mprobs
             'c':'-a-c-ctat-',
             'd':'-a-c-ctat-'})
         sm = Nucleotide(recode_gaps=True)
-        pc = sm.makeParamController(t)
+        lf = sm.makeParamController(t)
         #pc.setConstantLengths()
-        lf=pc.makeCalculator(al)
+        lf.setAlignment(al)
         #print lf.simulateAlignment(sequence_length=10)
         simulated = lf.simulateAlignment()
         self.assertEqual(len(simulated.getSeqNames()), 4)
@@ -438,8 +455,8 @@ motif  mprobs
             al = LoadSeqs(data={'a':'ggaatt','c':'cctaat'})
             t = LoadTree(treestring="(a,c);")
             sm = substitution_model.Dinucleotide(mprob_model='tuple')
-            pc = sm.makeParamController(t)
-            lf = pc.makeCalculator(al)
+            lf = sm.makeParamController(t)
+            lf.setAlignment(al)
             simalign = lf.simulateAlignment(exclude_internal=False,
                                             root_sequence=root_sequence)
             root = simalign.NamedSeqs['root']
@@ -454,15 +471,15 @@ motif  mprobs
         likelihood_function = self._makeLikelihoodFunction()
         self._setLengthsAndBetas(likelihood_function)
         tree = likelihood_function.getAnnotatedTree()
-        pc = self.submodel.makeParamController(tree)
-        lf = pc.makeCalculator(self.data)
+        lf = self.submodel.makeParamController(tree)
+        lf.setAlignment(self.data)
         self.assertEqual(lf.getParamValue("length", "Human"), 0.3)
         self.assertEqual(lf.getParamValue("beta", "Human"), 4.0)
     
     def test_set_par_all(self):
         likelihood_function = self._makeLikelihoodFunction()
-        likelihood_function.setpar("length", 4.0)
-        likelihood_function.setpar("beta", 6.0)
+        likelihood_function.setParamRule("length", value=4.0, is_const=True)
+        likelihood_function.setParamRule("beta", value=6.0, is_const=True)
         self.assertEqual(str(likelihood_function), \
 """Likelihood Function Table
 ======
@@ -531,12 +548,12 @@ motif    mprobs
     
     def test_getAnnotatedTree(self):
         likelihood_function = self._makeLikelihoodFunction()
-        likelihood_function.setpar("length", 4.0, edge="Human")
+        likelihood_function.setParamRule("length", value=4.0, edge="Human", is_const=True)
         result = likelihood_function.getAnnotatedTree()
         self.assertEqual(result.getNodeMatchingName('Human').params['length'], 4.0)
         self.assertEqual(result.getNodeMatchingName('Human').Length, 4.0)
     
-    def test_getstatsasdict(self):
+    def test_getparamsasdict(self):
         likelihood_function = self._makeLikelihoodFunction()
         likelihood_function.setName("TEST")
         self.assertEqual(str(likelihood_function),\
@@ -560,10 +577,7 @@ motif    mprobs
     A    0.2500
     G    0.2500
 ---------------""")
-        self.assertEqual(likelihood_function.getStatisticsAsDict(),
-{'edge.parent': {'NineBande': 'root', 'edge.1': 'root', 'DogFaced': 'root',
-         'Human': 'edge.0', 'edge.0': 'edge.1', 'Mouse': 'edge.1',
-         'HowlerMon': 'edge.0'},
+        self.assertEqual(likelihood_function.getParamValueDict(['edge']), {
  'beta': {'NineBande': 1.0, 'edge.1': 1.0,'DogFaced': 1.0, 'Human': 1.0,
       'edge.0': 1.0, 'Mouse': 1.0, 'HowlerMon': 1.0},
  'length': {'NineBande': 1.0,'edge.1': 1.0, 'DogFaced': 1.0, 'Human': 1.0,
@@ -576,8 +590,7 @@ motif    mprobs
         
         lf = submod.makeLikelihoodFunction(self.tree)
         lf.setAlignment(aln)
-        stats = lf.getStatisticsAsDict()
-        
+        stats = lf.getParamValueDict(['edge'], params=['length'])
     
     def test_constant_to_free(self):
         """excercise setting a constant param rule, then freeing it"""
@@ -586,10 +599,8 @@ motif    mprobs
         lf.setAlignment(self.data)
         lf.setParamRule('beta', is_const=True, value=2.0, 
                         edges=['NineBande', 'DogFaced'], is_clade=True)
-        lf.real_par_controller.makeCalculator()
         lf.setParamRule('beta', init=2.0, is_const=False,
                         edges=['NineBande', 'DogFaced'], is_clade=True)
-        lf.real_par_controller.makeCalculator()
     
 
 if __name__ == '__main__':
