@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 """Tests Numbers and Freqs objects, and their Unsafe versions.
 """
-
 from math import sqrt
+import numpy
 from cogent.util.unit_test import TestCase, main
 from cogent.maths.stats.util import SummaryStatistics, SummaryStatisticsError,\
         Numbers, UnsafeNumbers, Freqs, UnsafeFreqs, NumberFreqs, \
@@ -14,7 +14,7 @@ __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight", "Sandra Smit"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -395,7 +395,17 @@ class NumbersTestsI(object):
         self.assertEqual(self.empty.Mean, None)
         self.assertEqual(self.zero.Mean, 0)
         self.assertEqual(self.ints.Mean, 3)
-
+    
+    def test_NumberQuantiles(self):
+        """quantiles should be correct"""
+        num = self.ClassToTest(range(1,11))
+        self.assertFloatEqual(num.quantile(.1), 1.9)
+        self.assertFloatEqual(num.quantile(.2), 2.8)
+        self.assertFloatEqual(num.quantile(.25), 3.25)
+        self.assertFloatEqual(num.Median, 5.5)
+        self.assertFloatEqual(num.quantile(.75), 7.75)
+        self.assertFloatEqual(num.quantile(.77), 7.93)
+    
     def test_summarize(self):
         """Numbers summarize should return SummaryStatistics object"""
         self.assertEqual(self.ints.summarize(), SummaryStatistics(Mean=3,\
@@ -1800,7 +1810,25 @@ class NumberFreqsTestsI(object):
         self.assertFloatEqual(self.NumericDuplicated.StandardDeviation,1.108678)
         self.assertFloatEqual(self.PosNeg.StandardDeviation, 1.825742)
         self.assertFloatEqual(self.Constant.StandardDeviation, 0.0)
-
+    
+    def test_NumberFreqsQuantiles(self):
+        """quantiles should match Numbers, including Median"""
+        data={32: 60L, 33: 211L, 34: 141L, 35: 70L,
+             36: 26L, 10: 30L, 11: 5L, 18: 43L, 19: 10L,
+             21: 1L, 22: 1L, 23: 58L, 24: 12L, 25: 3L,
+             26: 74L, 27: 10L, 28: 77L, 29: 20L, 30: 102L,
+             31: 47L}
+        
+        nums = Numbers(NumberFreqs(data=data).expand())
+        number_freqs = self.ClassToTest()
+        number_freqs.update(data)
+        for quantile in numpy.arange(0.05, 0.96, 0.05):
+            num_q = nums.quantile(quantile)
+            num_f = number_freqs.quantile(quantile)
+            self.assertFloatEqual(num_f, num_q)
+        
+        self.assertFloatEqual(number_freqs.Median, nums.Median)
+    
     def test_normalize(self):
         """NumberFreqs should allow normalization on any type"""
         self.Empty.normalize()
