@@ -9,13 +9,17 @@ __all__ = ['test_aaindex',
            'test_dialign',
            'test_ebi',
            'test_fasta',
+           'test_fastq',
            'test_genbank',
+           'test_illumina_sequence',
            'test_locuslink',
            'test_mage',
            'test_meme',
            'test_msms',
            'test_ncbi_taxonomy',
            'test_nexus',
+           'test_pdb',
+           'test_structure',
            'test_phylip',
            'test_record',
            'test_record_finder',
@@ -35,7 +39,7 @@ __credits__ = ["Jeremy Widmann", "Catherine Lozuopone", "Gavin Huttley",
                     "Zongzhi Liu", "Jason Carnes", "Peter Maxwell",
                     "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
