@@ -7,35 +7,130 @@ from os import remove, rmdir
 from os.path import exists
 from cogent.app.util import get_tmp_filename
 from cogent.util.unit_test import TestCase, main
-from cogent.util.misc import iterable, max_index, min_index, \
-    flatten, is_iterable, is_char, is_char_or_noniterable,\
-    is_str_or_noniterable, not_list_tuple, list_flatten,\
-    recursive_flatten, unflatten, unzip, select, sort_order, find_all, \
-    find_many, unreserve,\
-    extract_delimited, caps_from_underscores,\
-    add_lowercase, InverseDict, InverseDictMulti, DictFromPos, DictFromFirst, \
-    DictFromLast, DistanceFromMatrix, PairsFromGroups, \
-    ClassChecker, Delegator, FunctionWrapper, \
-    ConstraintError, ConstrainedContainer,\
-    ConstrainedString, ConstrainedList, ConstrainedDict, \
-    MappedString, MappedList, MappedDict, \
-    generateCombinations, makeNonnegInt, \
-    NonnegIntError, revComp, not_none, get_items_except,\
-    NestedSplitter, curry, app_path, remove_files, get_random_directory_name
+from cogent.util.misc import (iterable, max_index, min_index,
+    flatten, is_iterable, is_char, is_char_or_noniterable,
+    is_str_or_noniterable, not_list_tuple, list_flatten,
+    recursive_flatten, unflatten, unzip, select, sort_order, find_all,
+    find_many, unreserve,
+    extract_delimited, caps_from_underscores,
+    add_lowercase, InverseDict, InverseDictMulti, DictFromPos, DictFromFirst,
+    DictFromLast, DistanceFromMatrix, PairsFromGroups,
+    ClassChecker, Delegator, FunctionWrapper,
+    ConstraintError, ConstrainedContainer,
+    ConstrainedString, ConstrainedList, ConstrainedDict,
+    MappedString, MappedList, MappedDict,
+    generateCombinations, makeNonnegInt,
+    NonnegIntError, reverse_complement, not_none, get_items_except,
+    NestedSplitter, curry, app_path, remove_files, get_random_directory_name,
+    revComp, parse_command_line_parameters, safe_md5, 
+    create_dir, handle_error_codes, identity, if_, deep_list, deep_tuple,
+    combinate,gzip_dump,gzip_load,recursive_flatten_old,getNewId,toString,
+    timeLimitReached)
 from numpy import array
+from time import clock, sleep
 
 __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight", "Amanda Birmingham", "Sandra Smit",
                     "Zongzhi Liu", "Peter Maxwell", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
 
 class UtilsTests(TestCase):
     """Tests of individual functions in utils"""
+    
+    def setUp(self):
+        """ """
+        self.files_to_remove = []
+        
+    def tearDown(self):
+        """ """
+        map(rmdir,self.files_to_remove)
+
+    def test_identity(self):
+        """should return same object"""
+        foo = [1,'a',lambda x: x]
+        exp = id(foo)
+        self.assertEqual(id(identity(foo)), exp)
+
+    def test_if_(self):
+        """implementation of c-like tertiary operator"""
+        exp = 'yay'
+        obs = if_(True, 'yay', 'nay')
+        self.assertEqual(obs, exp)
+        exp = 'nay'
+        obs = if_(False, 'yay', 'nay')
+        self.assertEqual(obs, exp)
+
+    def test_deep_list(self):
+        """should convert nested tuple to nested list"""
+        input = ((1,(2,3)),(4,5),(6,7))
+        exp = [[1,[2,3]],[4,5],[6,7]]
+        obs = deep_list(input)
+        self.assertEqual(obs, exp)
+
+    def test_deep_tuple(self):
+        """Should convert a nested list to a nested tuple"""
+        exp = ((1,(2,3)),(4,5),(6,7))
+        input = [[1,[2,3]],[4,5],[6,7]]
+        obs = deep_tuple(input)
+        self.assertEqual(obs, exp)
+
+    def test_combinate(self):
+        """Should return combinations"""
+        input = [1,2,3,4]
+        n = 2
+        exp = [[1,2],[1,3],[1,4],[2,3],[2,4],[3,4]]
+        obs = list(combinate(input, n))
+        self.assertEqual(obs, exp)
+
+    def test_recursive_flatten_old(self):
+        """Should flatten nested lists"""
+        input = [[[1,2],[3,[4,5]],[6,7]],8]
+        exp = [1,2,3,4,5,6,7,8]
+        obs = recursive_flatten_old(input)
+        self.assertEqual(obs, exp)
+        
+    def test_getNewId(self):
+        """should return a random 12 digit id"""
+        rand_f = lambda x: 1
+        obs = getNewId(rand_f=rand_f)
+        exp = '111111111111'
+        self.assertEqual(obs,exp)
+
+    def test_toString(self):
+        """should stringify an object"""
+        class foo(object):
+            def __init__(self):
+                self.bar = 5
+        exp = 'bar: 5'
+        obs = toString(foo())
+        self.assertEqual(obs, exp)
+
+    # this test and the code it tests is architecture dependent. that is not
+    # good.
+    #def test_timeLimitReached(self):
+    #    """should return true if timelimit has been reached, else return false"""
+    #    start = clock()
+    #    timelimit = .0002
+    #    exp = False
+    #    sleep(1)
+    #    obs = timeLimitReached(start, timelimit)
+    #    self.assertEqual(obs, exp)
+    #    sleep(1)
+    #    exp = True
+    #    obs = timeLimitReached(start, timelimit)
+    #    self.assertEqual(obs, exp)
+
+
+    def test_safe_md5(self):
+        """Make sure we have the expected md5"""
+        exp = '29fa8614a884efe6a5604b7ef1af7378'
+        obs = safe_md5(open('test_util/__init__.py'))
+        self.assertEqual(obs.hexdigest(),exp)
 
     def test_iterable(self):
         """iterable(x) should return x or [x], always an iterable result"""
@@ -118,6 +213,39 @@ class UtilsTests(TestCase):
             ['aa',[8,'cc','dd'], ['ee',['ff','gg']]],
             is_leaf=is_str_or_noniterable),
             ['aa',8,'cc','dd','ee','ff','gg'])
+            
+    def test_create_dir(self):
+        """create_dir creates dir and fails meaningful."""
+
+        tmp_dir_path = get_random_directory_name()
+        tmp_dir_path2 = get_random_directory_name(suppress_mkdir=True)
+        tmp_dir_path3 = get_random_directory_name(suppress_mkdir=True)
+
+        self.files_to_remove.append(tmp_dir_path)
+        self.files_to_remove.append(tmp_dir_path2)
+        self.files_to_remove.append(tmp_dir_path3)
+
+        # create on existing dir raises OSError if fail_on_exist=True
+        self.assertRaises(OSError, create_dir, tmp_dir_path,
+                          fail_on_exist=True)
+        self.assertEquals(create_dir(tmp_dir_path,
+                                     fail_on_exist=True,
+                                     handle_errors_externally=True), 1)
+
+        # return should be 1 if dir exist and fail_on_exist=False 
+        self.assertEqual(create_dir(tmp_dir_path, fail_on_exist=False), 1)
+
+        # if dir not there make it and return always 0
+        self.assertEqual(create_dir(tmp_dir_path2), 0)
+        self.assertEqual(create_dir(tmp_dir_path3, fail_on_exist=True), 0)
+
+    def test_handle_error_codes(self):
+        """handle_error_codes raises the right error."""
+
+        self.assertRaises(OSError, handle_error_codes, "test", False,1)
+        self.assertEqual(handle_error_codes("test", True, 1), 1)
+        self.assertEqual(handle_error_codes("test", False, 0), 0)
+        self.assertEqual(handle_error_codes("test"), 0)
 
     def test_not_list_tuple(self):
         """not_list_tuple(obj) should return False when obj is list or tuple"""
@@ -1254,66 +1382,71 @@ class makeNonnegIntTests(TestCase):
     #end test_makeNonnegInt_noncastable
 #end makeNonnegIntTests
 
-class revCompTests(TestCase):
-    """Tests of the public revComp function"""
+class reverse_complementTests(TestCase):
+    """Tests of the public reverse_complement function"""
     
-    def test_revComp_DNA(self):
-        """revComp should correctly return reverse complement of DNA"""
+    def test_reverse_complement_DNA(self):
+        """reverse_complement should correctly return reverse complement of DNA"""
         
         #input and correct output taken from example at 
         #http://bioweb.uwlax.edu/GenWeb/Molecular/Seq_Anal/
         #Reverse_Comp/reverse_comp.html
         user_input = "ATGCAGGGGAAACATGATTCAGGAC"
         correct_output = "GTCCTGAATCATGTTTCCCCTGCAT"
+        real_output = reverse_complement(user_input)
+        self.assertEquals(real_output, correct_output)
+        
+        # revComp is a pointer to reverse_complement (for backward 
+        # compatibility)
         real_output = revComp(user_input)
         self.assertEquals(real_output, correct_output)
-    #end test_revComp_DNA
+    #end test_reverse_complement_DNA
     
-    def test_revComp_RNA(self):
-        """revComp should correctly return reverse complement of RNA"""
+    def test_reverse_complement_RNA(self):
+        """reverse_complement should correctly return reverse complement of RNA"""
         
-        #input and correct output taken from test_revComp_DNA test,
+        #input and correct output taken from test_reverse_complement_DNA test,
         #with all Ts changed to Us
         user_input = "AUGCAGGGGAAACAUGAUUCAGGAC"
         correct_output = "GUCCUGAAUCAUGUUUCCCCUGCAU"
         
         #remember to use False toggle to get RNA instead of DNA
-        real_output = revComp(user_input, False)
+        real_output = reverse_complement(user_input, False)
         self.assertEquals(real_output, correct_output)        
-    #end test_revComp_RNA
+    #end test_reverse_complement_RNA
     
-    def test_revComp_caseSensitive(self):
-        """revComp should convert bases without changing case"""
+    def test_reverse_complement_caseSensitive(self):
+        """reverse_complement should convert bases without changing case"""
         
         user_input = "aCGtAcgT"
         correct_output = "AcgTaCGt"
-        real_output = revComp(user_input)
+        real_output = reverse_complement(user_input)
         self.assertEquals(real_output, correct_output) 
-    #end test_revComp_caseSensitive
+    #end test_reverse_complement_caseSensitive
     
-    def test_revComp_nonNucleicSeq(self):
-        """revComp should just reverse any chars but ACGT/U"""
+    def test_reverse_complement_nonNucleicSeq(self):
+        """reverse_complement should just reverse any chars but ACGT/U"""
         
         user_input = "BDeF"
         correct_output = "FeDB"
-        real_output = revComp(user_input)
+        real_output = reverse_complement(user_input)
         self.assertEquals(real_output, correct_output)   
-    #end test_revComp_nonNucleicSeq
+    #end test_reverse_complement_nonNucleicSeq
     
-    def test_revComp_emptySeq(self):
-        """revComp should return empty string if given empty sequence"""
+    def test_reverse_complement_emptySeq(self):
+        """reverse_complement should return empty string if given empty sequence"""
         
         #shouldn't matter whether in DNA or RNA mode
-        real_output = revComp("")
+        real_output = reverse_complement("")
         self.assertEquals(real_output, "") 
-    #end test_revComp_emptySeq
+    #end test_reverse_complement_emptySeq
     
-    def test_revComp_noSeq(self):
-        """revComp should return error if given no sequence argument"""
+    def test_reverse_complement_noSeq(self):
+        """reverse_complement should return error if given no sequence argument"""
         
-        self.assertRaises(TypeError, revComp)
-    #end test_revComp_noSeq
-#end revCompTests
+        self.assertRaises(TypeError, reverse_complement)
+    #end test_reverse_complement_noSeq
+#end reverse_complementTests
 
     def test_not_none(self):
         """not_none should return True if none of the items is None"""
@@ -1380,6 +1513,40 @@ class revCompTests(TestCase):
         self.assertEqual(app_path('ls'), '/bin/ls')
         self.assertEqual(app_path('lsxxyyx'), False)
 
+class CommandLineParserTests(TestCase):
+    
+    def test_parse_command_line_parameters(self):
+        """parse_command_line_parameters returns without error
+        
+            There is not a lot of detailed testing that can be done here,
+            so the basic functionality is tested.
+        """
+        option_parser, opts, args = parse_command_line_parameters(
+          script_description="My script",
+          script_usage=[('Print help','%prog -h','')],
+          version='1.0',help_on_no_arguments=False,
+          command_line_args=[])
+        self.assertEqual(len(args),0)
+        
+        d = {'script_description':"My script",\
+             'script_usage':[('Print help','%prog -h','')],\
+             'version':'1.0',
+             'help_on_no_arguments':False,
+             'command_line_args':[]}
+        option_parser, opts, args = parse_command_line_parameters(**d)
+        self.assertEqual(len(args),0)
+        
+        # allowing positional arguments functions as expected as does
+        # passing a positional argument
+        d = {'script_description':"My script",\
+             'script_usage':[('Print help','%prog -h','')],\
+             'version':'1.0',
+             'help_on_no_arguments':False,
+             'command_line_args':['hello'],
+             'disallow_positional_arguments':False}
+        option_parser, opts, args = parse_command_line_parameters(**d)
+        self.assertEqual(len(args),1)
+
 #run tests on command-line invocation
 
 if __name__ == '__main__':
